//
// Copyright (c) 2001, Junzo SATO. All rights reserverd.
//
package MathSocketTerminal;

// TCP/IP Connection

import java.awt.*;
import java.awt.event.*;

import java.io.*;
import java.net.*;
import java.util.*;

public class MathSocketTerminal {
	boolean isConnected = false;
	Socket clientSocket = null;
	String remoteServerAddr = "0.0.0.0";
	int remoteServerPort = 0;
	MathSocketSenderThread sender = null;
	MathSocketReceiverThread receiver = null;
	SocketListener listener = null;
	
	public MathSocketTerminal() {
		
	}
	
	public String Copyright() {
		return "Copyright (c) 2001, Junzo SATO. All rights reserved.";
	}
	
	public boolean connectToServer(String addr, int port) {
		if (isConnected) return true;// already connected
		
		// create client socket to the remote server

		// try to create client socket 
		// if the server is waiting to accept a client,
		// the session will be established
		remoteServerAddr = addr;
		remoteServerPort = port;
		try {
			// create socket to the server
			clientSocket = new Socket(remoteServerAddr, remoteServerPort);
		} catch (IOException e) {
			return false;
		}
		
		isConnected = true;
		return isConnected;
	}
	
	public void disconnectFromServer() {
		if (isConnected == false) return;
		
		if (receiver.isAlive()) {
			receiver.stop();
		}
		receiver = null;
		
		if (sender.isAlive()) {
			sender.stop();
		}
		sender = null;
		
		if (clientSocket != null) {
			try {
				clientSocket.close();
			} catch (IOException e) {
			
			}
			clientSocket = null;
		}
		
		isConnected = false;
	}
	
	public String startSession() {
		if (isConnected) {
			String str = negotiate();
			
			receiver = new MathSocketReceiverThread(clientSocket, listener);
			while (receiver.isAlive() == false) {}	
			sender = new MathSocketSenderThread(clientSocket);
			while (sender.isAlive() == false) {}
			
			return str;
		}
		
		return null;
	}
	
	public void sendRequest(String msg) {
		if (sender != null) {
			sender.sendRequest(msg);
		}
	}
	
	public void setSocketListener(SocketListener l) {
		if (receiver != null) {
			// session is started already. 
			listener = l;
			receiver.setSocketListener(l);
		} else {
			// session is not started yet.
			listener = l;
		}
	}

	public void echo(String str) {
		if (receiver != null) {
			receiver.echo(str);
		}
	}
	
	public String negotiate() {
		String str = null;
		
		// FOR TELNET 
		// Reference:
		// "Java Network Programming" by Tomihiro Odaka (Written in Japanese), Ohmsha, 1999 
		if (remoteServerPort == 23) {
			byte IAC = (byte)255;
			byte DONT = (byte)254;
			byte DO = (byte)253;
			byte WONT = (byte)252;
			byte WILL = (byte)251;
			try {
				BufferedInputStream in = new BufferedInputStream(clientSocket.getInputStream());
				OutputStream out = clientSocket.getOutputStream();
				byte[] token = new byte[3];
				while (true) {
					in.mark(token.length);
					if (in.available() >= token.length) {
						in.read(token);
						if (token[0] != IAC) {
							in.reset();
														
							// derive "login:" prompt
							int n = in.available();
							byte[] buff = new byte[n];
							in.read(buff);
							ByteArrayOutputStream ba = new ByteArrayOutputStream(n);
							ba.write(buff,0,n);
							str = ba.toString();
							str = str.trim();
							buff = null;
							
							return str;
						} else if (token[1] == DO) {
							token[1] = WONT;
							out.write(token);
							continue;
						}
					}
				}
			} catch (IOException e) {
			
			}
		}
		
		return str;
	}
	
	public String getFirstIncomingMessage() {
		long millisToWait = 5000;
		long start;

		InputStream inStream = null;
		BufferedInputStream in = null;
		try {
			inStream = clientSocket.getInputStream();
			in = new BufferedInputStream(clientSocket.getInputStream());
		} catch (IOException e) {
			return null;
		}

		int n = 0;
		String str = null;

		start = System.currentTimeMillis();
		while (true) {
			if (str == null) {
				try {
					n = in.available();
				} catch (IOException e1) {
					return null;
				}
				if (n <= 0) continue;
				
				byte[] buff = new byte[n];
				try {
					n = in.read(buff);
				} catch (IOException e2) {
					return null;
				}
				if (n <= 0) continue;
				
				ByteArrayOutputStream ba = new ByteArrayOutputStream(n);
				ba.write(buff,0,n);
				str = ba.toString();
				str = str.trim();
				buff = null;
				
				break;
			} else if (System.currentTimeMillis() - start > millisToWait) {
				// timeout
				break;
			}
		}
		
		return str;
	}
}